﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Flash: Zugriff auf Festspeicher            											//
//  _________________________________                                                   //
//                                                                                      //
//  Copyright (c) 2022 Reimar Grasbon                                                   //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//   Organisation Flashspeicher:                                                        //
//                                                                                      //
//                             +-------------------+ PGMLDR_SECTION_START (0h)          //
//                             |                   |                                    //
//                             |   Programmlader   |                                    //
//                             |                   |                                    //
//                             +-------------------+ APPL_SECTION_START                 //
//                             |                   |                                    //
//                             |                   |                                    //
//                             |                   |                                    //
//                             |                   |                                    //
//                             |    Applikation    |                                    //
//                             |                   |                                    //
//                             |                   |                                    //
//                             |                   |                                    //
//                             |                   |                                    //
//                             |-------------------| CY4ADR_SECTION_START               //
//                             |     BoardTag      |                                    //
//                             +-------------------+                                    //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TypeDefs.h"
#include "Conf.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
extern char FLASH_SIZE;				// Symbole definiert im

#define FLASH_MEMORY_SIZE  ((UInt32)(&FLASH_SIZE)) 				  // [Bytes]
#define FLASH_PAGE_BITS    CFG_FLASH_PAGE_BITS		              // [Bits]
#define FLASH_PAGE_SIZE    ((UInt32)(1 << FLASH_PAGE_BITS))		  // [Bytes]
#define FLASH_PAGE_COUNT   (FLASH_MEMORY_SIZE / FLASH_PAGE_SIZE)  // [Pages]

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef struct
{	UInt32 number;
} PageNr;

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
PageNr	FLASH_PageNumberOf(const void*);
void*	FLASH_PageBaseAddress(PageNr);
void    FLASH_ReadPage(PageNr, UInt08[FLASH_PAGE_SIZE]);
void    FLASH_ErasePage(PageNr);
bool    FLASH_WritePage(const UInt08[FLASH_PAGE_SIZE], PageNr);

//////////////////////////////////////////////////////////////////////////////////////////
